/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright  2000-2002 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "OgreOcclusionCullingSceneManager.h"
#include "OgreKdTreeSceneManager.h"
#include "OgreKdTerrainSceneManager.h"
#include <OgreRoot.h>
#include <OgreLogManager.h>

#include "OgreBvHierarchySceneManager.h"
#include "VisibilityManager.h"
#include "VisibilityEnvironment.h"
		
GtpVisibility::VisibilityEnvironment *visEnv;
GtpVisibility::VisibilityManager *visManager;

namespace Ogre {


OcclusionCullingSceneManagerFactory *occlusionCullingFactory;
KdTreeSceneManagerFactory *kdTreeFactory;
KdTerrainSceneManagerFactory *kdTerrainFactory;
BvHierarchySceneManagerFactory *bvHierarchyFactory;

//-----------------------------------------------------------------------
extern "C" void __declspec(dllexport) dllStartPlugin(void)
{
	// load visibility environment
	visEnv = new GtpVisibility::VisibilityEnvironment();
	visManager = new GtpVisibility::VisibilityManager(visEnv);

	// Create new scene manager
	occlusionCullingFactory = new OcclusionCullingSceneManagerFactory(visManager);
	kdTreeFactory = new KdTreeSceneManagerFactory(visManager);
	kdTerrainFactory = new KdTerrainSceneManagerFactory(visManager);
	bvHierarchyFactory = new BvHierarchySceneManagerFactory(visManager);

	// Construct listener manager singleton
	new TerrainPageSourceListenerManager();
	new KdTerrainPageSourceListenerManager();

	// Register
	Root::getSingleton().addSceneManagerFactory(occlusionCullingFactory);
	Root::getSingleton().addSceneManagerFactory(kdTreeFactory);
	Root::getSingleton().addSceneManagerFactory(kdTerrainFactory);
	Root::getSingleton().addSceneManagerFactory(bvHierarchyFactory);
}
//-----------------------------------------------------------------------
extern "C" void __declspec(dllexport) dllShutdownPlugin()
{
	Root::getSingleton().removeSceneManagerFactory(occlusionCullingFactory);
	Root::getSingleton().removeSceneManagerFactory(kdTreeFactory);
	Root::getSingleton().removeSceneManagerFactory(kdTerrainFactory);
	Root::getSingleton().removeSceneManagerFactory(bvHierarchyFactory);

	// destroy listener manager
	delete TerrainPageSourceListenerManager::getSingletonPtr();
	delete KdTerrainPageSourceListenerManager::getSingletonPtr();
}
//-----------------------------------------------------------------------
extern "C" void __declspec(dllexport) dllStopPlugin()
{
	//delete heightmapTerrainPageSource;
	// BUG: crashes on delete!!!! FIX this 
	delete occlusionCullingFactory;
	delete kdTreeFactory;
	delete kdTerrainFactory;
	delete bvHierarchyFactory;

	delete visManager;
	delete visEnv;
}



} //namespace Ogre
